@@ -11,6 +11,10 @@ warnings.simplefilter("ignore")
 
 import argparse
 import mitama.commands
+import gevent.monkey
+#import sqlalchemy_gevent
+
+gevent.monkey.patch_all()
 
 parser = argparse.ArgumentParser(description="Mitama command utilities")
 subparser = parser.add_subparsers()
@@ -19,9 +23,11 @@ init = subparser.add_parser("init", help="Initialize directory as a Mitama proje
 init.set_defaults(handler=commands.init)
 
 new = subparser.add_parser("new", help="Create new Mitama project directory")
+new.add_argument("name", help="new project's name")
 new.set_defaults(handler=commands.new)
 
 mkapp = subparser.add_parser("mkapp", help="Create new Mitam application")
+mkapp.add_argument("name", help="new app's name")
 mkapp.set_defaults(handler=commands.mkapp)
 
 version = subparser.add_parser("version", help="Check version code")
